@@ -6,20 +6,24 @@
 */
 
 #include "kateview_test.h"
-#include "moc_kateview_test.cpp"
 
+#include <KSyntaxHighlighting/Repository>
+#include <KSyntaxHighlighting/Theme>
 #include <katebuffer.h>
 #include <kateconfig.h>
 #include <katedocument.h>
 #include <kateview.h>
 #include <kateviewinternal.h>
+#include <ktexteditor/editor.h>
 #include <ktexteditor/message.h>
 #include <ktexteditor/movingcursor.h>
 
+#include <KLineEdit>
 #include <QScrollBar>
 #include <QSignalSpy>
 #include <QStandardPaths>
 #include <QTemporaryFile>
+#include <QTextLayout>
 #include <QtTestWidgets>
 
 #define testNewRow() (QTest::newRow(QStringLiteral("line %1").arg(__LINE__).toLatin1().data()))
@@ -487,8 +491,7 @@ void KateViewTest::testDragAndDrop()
     view->setSelection(Range(1, 0, 2, 0));
     QCOMPARE(view->selectionRange(), Range(1, 0, 2, 0));
 
-    (void)QTest::qWaitForWindowExposed(view);
-    QTest::qWait(100); // For whatever reason needed
+    QVERIFY(QTest::qWaitForWindowExposed(view));
 
     const QPoint startDragPos = internalView->mapFrom(view, view->cursorToCoordinate(KTextEditor::Cursor(1, 2)));
     const QPoint endDragPos = internalView->mapFrom(view, view->cursorToCoordinate(KTextEditor::Cursor(3, 0)));
@@ -770,4 +773,110 @@ void KateViewTest::testDdisplayRangeChangedEmitted()
     QVERIFY(spy.count() == 1);
 }
 
+void KateViewTest::testCrashOnPasteInOverwriteMode()
+{
+    // create doc with a few lines
+    KTextEditor::DocumentPrivate doc(false, false);
+    const QString testText(QStringLiteral("test1\ntest2\ntest3\n"));
+    doc.setText(testText);
+
+    // create view in overwrite mode
+    KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
+    view->doc()->config()->setOvr(true);
+
+    // simulate paste at end of document that did crash, bug 496612
+    view->setCursorPosition(KTextEditor::Cursor(2, 0));
+    view->paste(&testText);
+
+    // check result is ok
+    QCOMPARE(doc.text(), QStringLiteral("test1\ntest2\ntest1\ntest2\ntest3\n"));
+}
+
+void KateViewTest::testCommandBarSearchReplace()
+{
+    KTextEditor::DocumentPrivate doc(false, false);
+    const QString testText(QStringLiteral("hello\nhello\nabc\nhello"));
+    doc.setText(testText);
+
+    KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
+    view->switchToCmdLine();
+    const auto childs = view->findChildren<QWidget *>();
+    KLineEdit *lineEdit = nullptr;
+    for (auto c : childs) {
+        if (strcmp("KateCmdLineEdit", c->metaObject()->className()) == 0) {
+            lineEdit = qobject_cast<KLineEdit *>(c);
+            break;
+        }
+    }
+    QVERIFY(lineEdit);
+    lineEdit->returnKeyPressed(QStringLiteral("%s/hell/bell"));
+    QCOMPARE(doc.text(), QStringLiteral("bello\nbello\nabc\nbello"));
+}
+
+void KateViewTest::testSelectedTextFormats()
+{
+    KTextEditor::DocumentPrivate doc(false, false);
+    const QString testText(QStringLiteral("- []KOrganizer: add more new stuff to the Whatsnew dialog\n"));
+    QVERIFY(doc.setHighlightingMode(QStringLiteral("Markdown")));
+    doc.setText(testText);
+
+    KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
+    view->setCursorPosition({0, 6});
+    view->setSelection({0, 3, 0, 6});
+    QCOMPARE(view->selectionRange(), KTextEditor::Range(0, 3, 0, 6));
+
+    const QTextLayout *layout = view->textLayout({0, 0});
+    QVERIFY(layout);
+    const auto formats = layout->formats();
+    QCOMPARE(formats.size(), 4);
+
+    auto verifyFormat = [](const QTextLayout::FormatRange &f, int start, int length, bool bg, bool fg) {
+        QCOMPARE(f.start, start);
+        QCOMPARE(f.length, length);
+        QCOMPARE(f.format.hasProperty(QTextFormat::BackgroundBrush), bg);
+        QCOMPARE(f.format.hasProperty(QTextFormat::ForegroundBrush), fg);
+    };
+
+    verifyFormat(formats[0], 0, 2, /*hasBg=*/false, /*hasFg=*/true);
+    verifyFormat(formats[1], 2, 1, /*hasBg=*/false, /*hasFg=*/true);
+    verifyFormat(formats[2], 3, 3, /*hasBg=*/false, /*hasFg=*/true);
+    verifyFormat(formats[3], 6, 51, /*hasBg=*/false, /*hasFg=*/true);
+
+    // Test with block selection
+
+    const auto &repository = KTextEditor::Editor::instance()->repository();
+    auto lightTheme = repository.defaultTheme(KSyntaxHighlighting::Repository::LightTheme);
+    view->setConfigValue(QStringLiteral("theme"), lightTheme.name());
+
+    doc.setText(QStringLiteral("* hello\n* hello\n* hello\n"));
+    view->show();
+    view->setBlockSelection(true);
+    view->setCursorPosition({1, 4});
+    view->setSelection({0, 2, 1, 4});
+
+    // this test will break if someone removes "selected color" format normal textstyle
+    const QColor normalSelColor = lightTheme.selectedTextColor(KSyntaxHighlighting::Theme::TextStyle::Normal);
+
+    {
+        const QTextLayout *layoutLine0 = view->textLayout({0, 0});
+
+        auto fmt = layoutLine0->formats()[0];
+        QVERIFY(fmt.start == 0 && fmt.length == 2);
+
+        fmt = layoutLine0->formats()[1];
+        QVERIFY(fmt.start == 2 && fmt.length == 2 && fmt.format.foreground() == normalSelColor);
+
+        fmt = layoutLine0->formats()[2];
+        QVERIFY(fmt.start == 4 && fmt.length == 3 && fmt.format.foreground() != normalSelColor);
+
+        const QTextLayout *layoutLine1 = view->textLayout({1, 0});
+        fmt = layoutLine1->formats()[1];
+        QVERIFY(fmt.start == 2 && fmt.length == 2 && fmt.format.foreground() == normalSelColor);
+
+        fmt = layoutLine1->formats()[2];
+        QVERIFY(fmt.start == 4 && fmt.length == 3 && fmt.format.foreground() != normalSelColor);
+    }
+}
+
 // kate: indent-mode cstyle; indent-width 4; replace-tabs on;
+#include "moc_kateview_test.cpp"
